//  <C> Copyright 1995, by Emanuel G. Noik
//

#include <math.h>
#include <stdio.h>
#include <assert.h>
#include <tk.h>

#include <hygraph/Hygraph.h>
#include <util/MN_Dictionary.h>
#include <util/MN_Set.h>
#include <util/MN_Series.h>
#include <server/server.h>
#include <server/hgInfo.h>
#include "tclTkHInfo.h"
#include "tkHInfo.h"
#include "misc.h"
#include "util.h"

//------------------------------------------------------------------------------

extern TkHInfo* current_hi;

//------------------------------------------------------------------------------

COMMAND( grUpdate )
{
   char t;
   int h, wi = 100, hi = 100, n = 10, ty;

   CHK_CUR;
   CHK_ARGS( <, 3, "usage: grUpdate [v|a|b] Hist {Width} {Height} {NBars}" );

   if( sscanf( argv[2], "%d", &h ) != 1 ) return TCL_ERROR;
   if( sscanf( argv[1], "%c", &t ) != 1 ) return TCL_ERROR;
   if( (ty = Type( t )) < 0 ) RET_ERROR("bad type")
   if( argc > 3 ) if( sscanf( argv[3], "%d", &wi ) != 1 ) return TCL_ERROR;
   if( argc > 4 ) if( sscanf( argv[4], "%d", &hi ) != 1 ) return TCL_ERROR;
   if( argc > 5 ) if( sscanf( argv[5], "%d", &n  ) != 1 ) return TCL_ERROR;

   current_hi->UpdateHist( ty, h, wi, hi, n );
   return TCL_OK;
}

//------------------------------------------------------------------------------

COMMAND( grLink )
{
   char t;
   int on, ty;
   HGObject* o;

   CHK_CUR;
   CHK_ARGS( !=, 4, "usage: grLink [v|a] Id [0|1]" );

   if( sscanf( argv[1], "%c", &t  ) != 1 ) return TCL_ERROR;
   if( sscanf( argv[3], "%d", &on ) != 1 ) return TCL_ERROR;

   if( (ty = Type( t )) < 0 ) RET_ERROR("bad type")
   switch( ty )
   {
      case 0 : o = current_hi->hygraph->Vid( argv[2] ); break;
      case 1 : o = current_hi->hygraph->Aid( argv[2] ); break;
      case 2 : o = current_hi->hygraph->Bid( argv[2] ); break;
      default: RET_ERROR("bad_type") break;
   }

   current_hi->Link( ty, o, on );
   return TCL_OK;
}

//------------------------------------------------------------------------------

// Turn histogram bar and all selected elements ON or OFF.
//
COMMAND( grPaint )
{
   char t;
   int h, b, on, ty;

   CHK_CUR;
   CHK_ARGS( !=, 5, "usage: grPaint [v|a|b] Hist Bar [0|1]" );

   if( sscanf( argv[1], "%c", &t  ) != 1 ) return TCL_ERROR;
   if( sscanf( argv[2], "%d", &h  ) != 1 ) return TCL_ERROR;
   if( sscanf( argv[3], "%d", &b  ) != 1 ) return TCL_ERROR;
   if( sscanf( argv[4], "%d", &on ) != 1 ) return TCL_ERROR;

   if( (ty = Type( t )) < 0 ) RET_ERROR("bad type")

   current_hi->Paint( ty, h, b, on );
   return TCL_OK;
}

//------------------------------------------------------------------------------

COMMAND( grMask )
{
   char t;
   int h, b, m, ty;

   CHK_CUR;
   CHK_ARGS( !=, 5, "usage: grMask [v|a|b] Hist Bar Mode" );

   if( sscanf( argv[1], "%c", &t ) != 1 ) return TCL_ERROR;
   if( sscanf( argv[2], "%d", &h ) != 1 ) return TCL_ERROR;
   if( sscanf( argv[3], "%d", &b ) != 1 ) return TCL_ERROR;
   if( sscanf( argv[4], "%d", &m ) != 1 ) return TCL_ERROR;

   if( (ty = Type( t )) < 0 ) RET_ERROR("bad type")

   current_hi->Mask( ty, h, b, m  );
   return TCL_OK;
}

//------------------------------------------------------------------------------

COMMAND( grMap )
{
   char t;
   int i, ty;
   PAttribute pa;

   CHK_CUR;
   CHK_ARGS( !=, 4, "usage: grMap [v|a|b] Index Attr" );

   if( sscanf( argv[1], "%c", &t ) != 1 ) RET_ERROR("bad type")
   if( sscanf( argv[2], "%d", &i ) != 1 ) RET_ERROR("bad index")

   if( (ty = Type( t )) < 0 ) RET_ERROR("bad type")

   for( int j = 0; attributes[j].label != NIL; j++ )
      if( strcmp( argv[3], attributes[j].label ) == 0 )
      {
         pa = attributes[j].attr;
         return (current_hi->MapAttr( ty, i, pa, argv[3] ) == ERR_OK)
            ? TCL_OK : TCL_ERROR;
      }

   Tcl_ResetResult( interp );

   RET_ERROR("bad attribute")
}

//------------------------------------------------------------------------------

int GrStat_Init( Tcl_Interp *interp )
{
   Tcl_CreateCommand( interp, "grUpdate",  grUpdate,  (ClientData)NULL,
      (Tcl_CmdDeleteProc*)NULL);
   Tcl_CreateCommand( interp, "grPaint",   grPaint,   (ClientData)NULL,
      (Tcl_CmdDeleteProc*)NULL);
   Tcl_CreateCommand( interp, "grLink",    grLink,    (ClientData)NULL,
      (Tcl_CmdDeleteProc*)NULL);
   Tcl_CreateCommand( interp, "grMask",    grMask,    (ClientData)NULL,
      (Tcl_CmdDeleteProc*)NULL);
   Tcl_CreateCommand( interp, "grMap",     grMap,     (ClientData)NULL,
      (Tcl_CmdDeleteProc*)NULL);
   return TCL_OK;
}

//------------------------------------------------------------------------------
