#include <math.h>
#include <stdio.h>
#include <tk.h>
#include "util.h"

//------------------------------------------------------------------------------

extern "C" double drand48( void );

COMMAND( uRandom )
{
  int i, integer = 0;
  long int seed = 1;
  double min =  0;
  double max = 10;
  double value;

  for( i = 1; i < argc; i++ )
  {
    if(      strcmp( argv[i], "-i"   ) == 0 ) { integer = 1; }
    else if( strcmp( argv[i], "-s"   ) == 0 ) { seed = atoi( argv[i+1] ); i++;}
    else if( strcmp( argv[i], "-min" ) == 0 ) { min = atof( argv[i+1] ); i++;}
    else if( strcmp( argv[i], "-max" ) == 0 ) { max = atof( argv[i+1] ); i++;}
  }

  if( min >= max )
  {
    min =  0;
    max = 10;
  }

  value = min + ( max - min ) * drand48();
  if( integer )
    sprintf( interp->result, "%d", (int)value );
  else
    sprintf( interp->result, "%g", value );

  return TCL_OK;
}

//------------------------------------------------------------------------------

MATH_FUNC( uMinMax )
{
   float l = args[0].doubleValue;
   float r = args[1].doubleValue;
   result->type = TCL_DOUBLE;
   result->doubleValue = (int)data == 0 ? (l < r ? l : r) : (l > r ? l : r);
   return TCL_OK;
}

//------------------------------------------------------------------------------


COMMAND( uTmpNam )
{
   CHK_ARGS( !=, 1, "usage: uTmpNam" );
   sprintf( interp->result, "%s", tmpnam( 0 ) );
   return TCL_OK;
}

//------------------------------------------------------------------------------

COMMAND( uClrStr )
{
   CHK_ARGS( !=, 4, "usage: uClrStr R G B" );
   sprintf( interp->result, "#%02x%02x%02x",
      atoi( argv[1] ), atoi( argv[2] ), atoi( argv[3] ) );
   return TCL_OK;
}

COMMAND( uClrTriple )
{
   CHK_ARGS( !=, 2, "usage: uClrTriple ClrStr" );
   int r = 255, g = 255, b = 255;
   sscanf( argv[1], "#%02x%02x%02x", &r, &g, &b );
   sprintf( interp->result, "%d %d %d", r, g, b );
   return TCL_OK;
}

COMMAND( uClrInv )
{
   CHK_ARGS( !=, 2, "usage: uClrInv RGBStr" );
   int r = 255, g = 255, b = 255;
   sscanf( argv[1], "#%02x%02x%02x", &r, &g, &b );
   sprintf( interp->result, "#%02x%02x%02x", 255 - r, 255 - g, 255 - b );
   return TCL_OK;
}

//------------------------------------------------------------------------------

int GrUtil_Init( Tcl_Interp* interp )
{
   Tcl_CreateCommand( interp, "uRandom", uRandom, (ClientData)NULL,
      (Tcl_CmdDeleteProc*)NULL);

   Tcl_ValueType a_dd[] = { TCL_DOUBLE, TCL_DOUBLE };
   Tcl_CreateMathFunc( interp, "uMin", 2, a_dd, uMinMax, (ClientData)0 );
   Tcl_CreateMathFunc( interp, "uMax", 2, a_dd, uMinMax, (ClientData)1 );

   Tcl_CreateCommand( interp, "uTmpNam",    uTmpNam,    (ClientData)NULL,
      (Tcl_CmdDeleteProc*)NULL);
   Tcl_CreateCommand( interp, "uClrStr",    uClrStr,    (ClientData)NULL,
      (Tcl_CmdDeleteProc*)NULL);
   Tcl_CreateCommand( interp, "uClrTriple", uClrTriple, (ClientData)NULL,
      (Tcl_CmdDeleteProc*)NULL);
   Tcl_CreateCommand( interp, "uClrInv",    uClrInv,    (ClientData)NULL,
      (Tcl_CmdDeleteProc*)NULL);
   return TCL_OK;
}

//------------------------------------------------------------------------------
